@@ -9,7 +9,7 @@
 import logging
 
 from .argument import Argument
-from .transformers import AvailabilityChecker
+from .transformers import AvailabilityChecker, KwargsAliasTransformer
 
 LOG = logging.getLogger(__name__)
 
@@ -21,6 +21,8 @@ class InputManager:
     ):
         self.decorators = decorators
         self._pipeline = None
+        self.alias_argument = None
+        self.alias_arguments = []
         self.availabilities = []
         self.arguments = {}
 
@@ -42,6 +44,9 @@ class InputManager:
             self.arguments[decorator.name] = Argument(decorator.name)
         self.arguments[decorator.name].normalize = decorator.kwargs
 
+    def register_alias_argument(self, decorator):
+        self.alias_arguments.append(decorator)
+
     def build_pipeline(self):
         self._pipeline = []
         LOG.debug("Building arguments from decorators:\n %s", self.decorators)
@@ -53,6 +58,10 @@ class InputManager:
 
         self.arguments = list(self.arguments.values())
 
+        for a in self.alias_arguments:
+            transform = KwargsAliasTransformer(a)
+            self._pipeline.append(transform)
+
         for a in self.arguments:
             a.add_alias_transformers(self._pipeline)
 
@@ -69,8 +78,23 @@ class InputManager:
         for a in self.arguments:
             a.add_format_transformers(self._pipeline)
 
+    def apply_to_kwargs_before_default(self, kwargs):
+        LOG.debug(
+            f"Apply pipeline to kwargs before resolving default values: {safe_to_str(kwargs)}"
+        )
+        for t in self.pipeline:
+            if hasattr(t, "name"):
+                LOG.debug(f" - {t.name}: apply {t}.")
+            else:
+                LOG.debug(f" - apply {t}.")
+
+            kwargs = t.execute_before_default(kwargs)
+            LOG.debug(f"       kwargs: {safe_to_str(kwargs)}")
+
+        return kwargs
+
     def apply_to_kwargs(self, kwargs):
-        LOG.debug(f"Apply pipeline to kwargs: {kwargs}")
+        LOG.debug(f"Apply pipeline to kwargs: {safe_to_str(kwargs)}")
         for t in self.pipeline:
             if hasattr(t, "name"):
                 LOG.debug(f" - {t.name}: apply {t}.")
@@ -78,7 +102,7 @@ class InputManager:
                 LOG.debug(f" - apply {t}.")
 
             kwargs = t.execute(kwargs)
-            LOG.debug(f"       kwargs: {kwargs}")
+            LOG.debug(f"       kwargs: {safe_to_str(kwargs)}")
 
         return kwargs
 
@@ -123,6 +147,9 @@ class InputManager:
         from climetlab.arguments.args_kwargs import ArgsKwargs
 
         args_kwargs = ArgsKwargs(args, kwargs, func=func)
+        LOG.debug("Applying decorator stack (before default) to: %s %s", args, kwargs)
+        args_kwargs.kwargs = self.apply_to_kwargs_before_default(args_kwargs.kwargs)
+
         args_kwargs.add_default_values_and_kwargs()
         self.consolidate_defaults(args_kwargs.defaults)
 
@@ -139,3 +166,14 @@ class InputManager:
 
 def normaliser(*args, **kwargs):
     return Argument("no-name", *args, **kwargs)
+
+
+def safe_to_str(dic):
+    lst = []
+    for k, v in dic.items():
+        try:
+            v = str(v)
+        except:  # noqa: E722
+            v = "..."
+        lst.append(f"{k}={v}")
+    return ",".join(lst)
