@@ -26,6 +26,9 @@ class Action:
     def execute(self, kwargs):
         raise NotImplementedError()
 
+    def execute_before_default(self, kwargs):
+        return kwargs
+
     def __repr__(self) -> str:
         return f"{self.__class__}"
 
@@ -156,3 +159,41 @@ class AvailabilityChecker(Action):
             if line:
                 txt += "\n    " + line
         return txt
+
+
+class KwargsAliasTransformer(Action):
+    def __init__(self, alias_argument) -> None:
+        self.aliases = alias_argument.kwargs
+
+    def execute(self, kwargs):
+        return kwargs
+
+    def execute_before_default(self, kwargs):
+        if kwargs:
+            LOG.debug("Transforming kwargs names with aliases for %s", kwargs)
+        assert isinstance(kwargs, dict), kwargs
+        new_kwargs = {}
+        for k, v in kwargs.items():
+            new_k = self.reversed_aliases.get(k, k)
+            assert (
+                new_k not in new_kwargs
+            ), f"Error: Multiple values were given for aliased arguments: with '{k}' and '{new_k}'."
+            new_kwargs[new_k] = v
+        return new_kwargs
+
+    @property
+    def reversed_aliases(self):
+        reversed = {}
+        for target, aliases in self.aliases.items():
+            for alias in aliases:
+                assert alias not in reversed, (
+                    "Error: Multiple target value for alias "
+                    f" argument '{alias}': '{target}' and '{reversed[alias]}'"
+                )
+                reversed[alias] = target
+        return reversed
+
+    def __repr__(self) -> str:
+        txt = "KwargsAlias:"
+        txt += ",".join([f"{v}->{k}" for k, v in self.aliases.items()])
+        return txt
