@@ -6,9 +6,13 @@
 # granted to it by virtue of its status as an intergovernmental organisation
 # nor does it submit to any jurisdiction.
 #
-from .reader import GRIBReader
+import logging
+
+LOG = logging.getLogger(__name__)
 
 
 def reader(source, path, magic=None, deeper_check=False):
     if magic is None or magic[:4] == b"GRIB":
+        from .reader import GRIBReader
+
         return GRIBReader(source, path)
