@@ -18,6 +18,7 @@ from importlib import import_module
 import entrypoints
 from termcolor import colored
 
+from .availability import AvailabilityCmd
 from .benchmark import BenchmarkCmd
 from .cache import CacheCmd
 from .check import CheckCmd
@@ -62,6 +63,7 @@ class CliMetLabApp(
     GribCmd,
     BenchmarkCmd,
     GribInfoCmd,
+    AvailabilityCmd,
     *get_plugins(),
 ):
     # intro = 'Welcome to climetlab. Type ? to list commands.\n'
