@@ -64,10 +64,9 @@ EXTENSIONS = {
 }
 
 
-class CDSRetriever(FileSource):
-
+class CdsRetriever(FileSource):
     sphinxdoc = """
-    CDSRetriever
+    CdsRetriever
     """
 
     def __init__(self, dataset, *args, **kwargs):
@@ -90,7 +89,6 @@ class CDSRetriever(FileSource):
             self.path = [self._retrieve(dataset, r) for r in requests]
         else:
             with SoftThreadPool(nthreads=nthreads) as pool:
-
                 futures = [pool.submit(self._retrieve, dataset, r) for r in requests]
 
                 iterator = (f.result() for f in futures)
@@ -109,6 +107,13 @@ class CDSRetriever(FileSource):
     @normalize("date", "date-list(%Y-%m-%d)")
     @normalize("area", "bounding-box(list)")
     def requests(self, **kwargs):
+        # TODO: move these 5 lines into @normalize
+        if "year" in kwargs:
+            if "month" not in kwargs:
+                kwargs["month"] = [f"{i+1:02}" for i in range(0, 12)]
+            if "day" not in kwargs:
+                kwargs["day"] = [f"{i+1:02}" for i in range(0, 31)]
+
         split_on = kwargs.pop("split_on", None)
         if split_on is None or not isinstance(kwargs.get(split_on), (list, tuple)):
             return [kwargs]
@@ -145,4 +150,4 @@ class CDSRetriever(FileSource):
         )
 
 
-source = CDSRetriever
+source = CdsRetriever
