@@ -39,10 +39,14 @@ class StandaloneMarsClient:
                 f.write(req_str + "\n")
             LOG.debug(f"Sending Mars request: '{req_str}'")
 
-            subprocess.run([self.EXE, filename], check=True)
+            subprocess.run(
+                [self.EXE, filename],
+                env=dict(os.environ, MARS_AUTO_SPLIT_BY_DATES="1"),
+                check=True,
+            )
 
 
-class MARSRetriever(ECMWFApi):
+class MarsRetriever(ECMWFApi):
     def service(self):
         if SETTINGS.get("use-standalone-mars-client-when-available"):
             if os.path.exists(StandaloneMarsClient.EXE):
@@ -61,4 +65,4 @@ class MARSRetriever(ECMWFApi):
             raise
 
 
-source = MARSRetriever
+source = MarsRetriever
