@@ -13,7 +13,30 @@ from collections import defaultdict
 
 
 def bytes(n):
-    u = ["", " KiB", " MiB", " GiB", " TiB", " PiB", "EiB", "ZiB", "YiB"]
+    """
+    >>> bytes(4096)
+    '4 KiB'
+    >>> bytes(4000)
+    '3.9 KiB'
+    """
+
+    u = ["", " KiB", " MiB", " GiB", " TiB", " PiB", " EiB", " ZiB", " YiB"]
+    i = 0
+    while n >= 1024:
+        n /= 1024.0
+        i += 1
+    return "%g%s" % (int(n * 10 + 0.5) / 10.0, u[i])
+
+
+def base2(n):
+    """
+    >>> base2(4096)
+    '4K'
+    >>> base2(4000)
+    '3.9K'
+    """
+
+    u = ["", "K", "M", "G", "T", " P", "E", "Z", "Y"]
     i = 0
     while n >= 1024:
         n /= 1024.0
@@ -38,7 +61,6 @@ def _plural(count):
 
 
 def seconds(seconds):
-
     if isinstance(seconds, datetime.timedelta):
         seconds = seconds.total_seconds()
 
@@ -244,14 +266,12 @@ def string_distance(s, t):
 
 
 def did_you_mean(word, vocabulary):
-
     distance, best = min((string_distance(word, w), w) for w in vocabulary)
     # if distance < min(len(word), len(best)):
     return best
 
 
 def dict_to_human(query):
-
     lst = [f"{k}={v}" for k, v in sorted(query.items())]
 
     return list_to_human(lst)
