@@ -16,7 +16,6 @@ class NumpyArrayWrapper(Wrapper):
         self.data = data
 
     def plot_map(self, backend):
-
         wrapper = get_wrapper(backend.option("metadata"))
 
         if hasattr(wrapper, "plot_numpy"):
@@ -37,7 +36,10 @@ class NumpyArrayWrapper(Wrapper):
         )
 
     def to_datetime_list(self):
-        return [datetime.datetime.fromtimestamp(x * 1e-9) for x in self.data.tolist()]
+        return [
+            datetime.datetime.fromtimestamp(x * 1e-9, tz=datetime.timezone.utc)
+            for x in self.data.tolist()
+        ]
 
 
 def wrapper(data, *args, **kwargs):
