@@ -18,9 +18,16 @@ LATLON = (
 )
 
 
+class PandasSeriesWrapper(Wrapper):
+    def __init__(self, series, **kwargs):
+        self.series = series
+
+    def to_datetime_list(self):
+        return sorted(set(self.series.values))
+
+
 class PandasFrameWrapper(Wrapper):
     def __init__(self, frame, **kwargs):
-
         self.frame = frame
 
         self.lat = "cannot-find-latitude-column"
@@ -41,7 +48,6 @@ class PandasFrameWrapper(Wrapper):
         backend.plot_graph_pandas(self.frame, self.time, column)
 
     def plot_map(self, backend):
-
         column = backend.option("column", self.lat)
 
         north, west, south, east = self.bounding_box()
@@ -56,7 +62,6 @@ class PandasFrameWrapper(Wrapper):
         backend.plot_pandas(self.frame, self.lat, self.lon, column)
 
     def bounding_box(self):
-
         north = self.frame[self.lat].max()
         south = self.frame[self.lat].min()
 
@@ -99,4 +104,7 @@ def wrapper(data, *args, **kwargs):
     if isinstance(data, pd.DataFrame):
         return PandasFrameWrapper(data, *args, **kwargs)
 
+    if isinstance(data, pd.Series):
+        return PandasSeriesWrapper(data, *args, **kwargs)
+
     return None
